"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodePoolWorker = void 0;
const node_util_1 = require("node:util");
const microservice_1 = require("@jfrog-pipelines/microservice");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const modules_1 = require("./modules");
class NodePoolWorker {
    constructor(details, store) {
        this.details = details;
        this.store = store;
        this.logger = logger_1.LoggerFactory.getLogger('nodePoolWorker');
        this.nodePoolService = new modules_1.NodePoolService();
        this.systemProperties = configuration_1.ConfigFactory.config;
    }
    async process(message) {
        const publisher = new amqp_client_1.AmqpPublisher(amqp_client_1.AmqpFactory.getClient(), this.systemProperties.amqpExchange, amqp_client_1.publisherOptions, this.logger);
        if (message) {
            const poolID = message.nodePoolId;
            const systemInstanceId = this.store.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER).systemServiceInstancesId;
            const lockService = new modules_1.LockService(systemInstanceId);
            const lockName = [...this.details.name, '-nodePools-', ...String(poolID)].join('');
            try {
                const nodePool = await this.nodePoolService.getNodePool(poolID);
                if (!nodePool) {
                    this.logger.info(`No valid node pool found for id ${poolID}`);
                    return;
                }
                if (!nodePool.isOnDemand) {
                    this.logger.debug(`Node Pool is not on demand, wont continue with execution`);
                    return;
                }
                const collector = new modules_1.NodeCollector(publisher);
                const provisioner = new modules_1.NodeProvisioner(publisher);
                const lock = await lockService.obtainLock({
                    name: lockName,
                    lockTimeoutInMs: modules_1.LockTimeoutInMs
                });
                if (!lock) {
                    this.logger.warn(`Failed to obtain lock for node pool id ${poolID}`);
                    return;
                }
                await provisioner.provision(nodePool);
                await collector.deProvision(nodePool);
            }
            catch (error) {
                this.logger.error(`Failed to process node pool id ${poolID} with err ${(0, node_util_1.inspect)(error)}`);
            }
            finally {
                await lockService.releaseLock(lockName);
            }
        }
    }
}
exports.NodePoolWorker = NodePoolWorker;
//# sourceMappingURL=NodePoolWorker.js.map