"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const NodeCollector_1 = require("../NodeCollector");
const NodePoolService_1 = require("../nodePool/NodePoolService");
const NodeService_1 = require("../node/NodeService");
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const client = new amqp_client_1.AmqpClient('', '', { amqpUrl: '' }, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publisher = new amqp_client_1.AmqpPublisher(client, '', amqp_client_1.publisherOptions, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce();
jest.spyOn(configuration_1.ConfigFactory, 'config', 'get').mockImplementation(() => {
    return { rtRouterJoinKey: 'abcd1234' };
});
describe('Test NodeCollector #deProvision', () => {
    const nodeCollector = new NodeCollector_1.NodeCollector(publisher);
    const nodePool = { id: 1, name: 'testpool', projectId: 1, isCacheEnabled: true, isOnDemand: true };
    const getNodePoolMock = jest.spyOn(NodePoolService_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue(nodePool);
    afterEach(() => {
        publishMessageMock.mockClear();
        getNodePoolMock.mockClear();
    });
    const STATUS_UPDATED_TIMEOUT = 15 * 60000;
    const PROVISIONED_DATE_TIME = 15 * 24 * 60 * 60000;
    const nodes = [
        { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
        {
            friendlyName: 'test2',
            projectId: 1,
            nodePoolId: 1,
            statusCode: NodeService_1.NodeStatus.success,
            statusLastUpdatedAt: new Date(Date.now() - STATUS_UPDATED_TIMEOUT),
            provisionedAt: new Date(Date.now() - PROVISIONED_DATE_TIME)
        }
    ];
    const filterActiveNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'filterActiveNodes').mockImplementation((nodes) => nodes.filter((node) => {
        return node.statusCode != NodeService_1.NodeStatus.cached;
    }));
    jest.spyOn(NodeService_1.NodeService.prototype, 'filterIdleNodes').mockImplementation((nodePool, nodes) => nodes.filter((node) => {
        return node.statusCode === NodeService_1.NodeStatus.success;
    }));
    jest
        .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToCachedState')
        .mockResolvedValue({ friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached });
    it('should deprovison cached nodes and not change deleted nodes', async () => {
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        filterActiveNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should  not change deleted nodes based on nodeStopSetting', async () => {
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopSetting: '' };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        filterActiveNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should  delete node based on nodeStopIntervalDays', async () => {
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopIntervalDays: 1, nodeStopSetting: 'nodeStopIntervalDays' };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        filterActiveNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should  not delete node based on nodeProvisionedAt', async () => {
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopIntervalDays: 20, nodeStopSetting: 'nodeStopIntervalDays' };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        filterActiveNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should  delete node based on nodeProvisionedAt', async () => {
        const nodes = [
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
            {
                friendlyName: 'test2',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date(),
                provisionedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopIntervalDays: 20, nodeStopSetting: 'nodeStopIntervalDays' };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        filterActiveNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
});
//# sourceMappingURL=NodeCollector.test.js.map