"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeService = exports.NodeActions = exports.NodeAccessorID = exports.ActiveNodeStatus = exports.NodeStatus = void 0;
const node_util_1 = require("node:util");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const logger_1 = require("@jfrog-pipelines/logger");
const access_nodejs_client_1 = require("@jfrog/access-nodejs-client");
const configuration_1 = require("@jfrog-pipelines/configuration");
const NodeDao_1 = require("./NodeDao");
var NodeStatus;
(function (NodeStatus) {
    NodeStatus[NodeStatus["queued"] = 4000] = "queued";
    NodeStatus[NodeStatus["processing"] = 4001] = "processing";
    NodeStatus[NodeStatus["success"] = 4002] = "success";
    NodeStatus[NodeStatus["failure"] = 4003] = "failure";
    NodeStatus[NodeStatus["waiting"] = 4005] = "waiting";
    NodeStatus[NodeStatus["cancelled"] = 4006] = "cancelled";
    NodeStatus[NodeStatus["unstable"] = 4007] = "unstable";
    NodeStatus[NodeStatus["skipped"] = 4008] = "skipped";
    NodeStatus[NodeStatus["timeout"] = 4009] = "timeout";
    NodeStatus[NodeStatus["stopped"] = 4010] = "stopped";
    NodeStatus[NodeStatus["deleted"] = 4011] = "deleted";
    NodeStatus[NodeStatus["cached"] = 4012] = "cached";
    NodeStatus[NodeStatus["cancelling"] = 4013] = "cancelling";
    NodeStatus[NodeStatus["timingout"] = 4014] = "timingout";
})(NodeStatus = exports.NodeStatus || (exports.NodeStatus = {}));
const MS = 60 * 1000;
exports.ActiveNodeStatus = [
    NodeStatus.success,
    NodeStatus.waiting,
    NodeStatus.queued,
    NodeStatus.processing,
    NodeStatus.cached
];
exports.NodeAccessorID = '1';
var NodeActions;
(function (NodeActions) {
    NodeActions[NodeActions["CREATE"] = 10] = "CREATE";
    NodeActions[NodeActions["CACHE"] = 20] = "CACHE";
    NodeActions[NodeActions["START"] = 30] = "START";
    NodeActions[NodeActions["DELETE"] = 40] = "DELETE";
})(NodeActions = exports.NodeActions || (exports.NodeActions = {}));
const oneYearInSeconds = 31536000;
const defaultApplcationHost = `localhost`;
class NodeService {
    constructor(publisher) {
        this.publisher = publisher;
        this.logger = logger_1.LoggerFactory.getLogger('NodeService');
        this.nodeDao = new NodeDao_1.NodeDao();
        this.properties = configuration_1.ConfigFactory.config;
        const accessClientPayload = {
            joinKey: this.properties.rtRouterJoinKey,
            routerBaseUrl: `${defaultApplcationHost}:`.concat(`${this.properties.rtRouterAccessPort}`),
            accessBaseUrl: `http://${defaultApplcationHost}:`.concat(`${this.properties.rtRouterAccessPort}`, '/access'),
            accessGrpcBaseUrl: `${defaultApplcationHost}:`.concat(`${this.properties.rtRouterAccessPort}`)
        };
        this.accessClient = new access_nodejs_client_1.AccessClient(accessClientPayload);
    }
    async createNode(attribute) {
        const node = await this.nodeDao.create(attribute);
        this.logger.debug(`Node with node pool id ${attribute.nodePoolId} created successfully with response ${(0, node_util_1.inspect)(node)}`);
        const nodeToken = await this.getNodeAccessToken(node);
        await this.updateNodeBootStrapToken(node, nodeToken);
        await this.publisher.publishMessage('core.marshaller', { action: NodeActions.CREATE, nodeId: node.id });
        await this.sendWWWSignal(node);
        return node;
    }
    async getRunningNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({})
            .where(`nodePoolId`, poolID)
            .where('statusCode', exports.ActiveNodeStatus)
            .where('stopRequested', 'false');
        return this.nodeDao.findAll(queryBuilder.build());
    }
    filterActiveNodes(nodes) {
        const activeNodes = nodes.filter((node) => {
            return node.statusCode !== NodeStatus.cached;
        });
        return activeNodes;
    }
    filterIdleNodes(nodePool, nodes) {
        return nodes.filter((node) => {
            const nodeLastUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
            return (node.statusCode !== NodeStatus.processing &&
                Date.now() - nodeLastUpdatedAt >= nodePool.nodeIdleIntervalInMins * MS);
        });
    }
    filterCachedNodes(nodes) {
        const cachedNodes = nodes.filter((node) => {
            return node.statusCode === NodeStatus.cached;
        });
        return cachedNodes;
    }
    async applyNodeToStoppedState(node) {
        return this.applyNodeState(node, NodeActions.DELETE, NodeStatus.stopped);
    }
    async applyNodeToCachedState(node) {
        return this.applyNodeState(node, NodeActions.CACHE, NodeStatus.cached);
    }
    async applyNodeWaitingState(node) {
        return this.applyNodeState(node, NodeActions.START, NodeStatus.waiting);
    }
    async applyNodeState(node, action, status) {
        try {
            const updateResponse = await this.nodeDao.updateByInstance(node, { statusCode: status });
            await this.publisher.publishMessage('core.marshaller', {
                action: action,
                projectId: node.projectId,
                nodeId: node.id
            });
            return updateResponse;
        }
        catch (error) {
            this.logger.warn(`Failed to applyNodeState to ${status} for node id ${node.id} due to err ${(0, node_util_1.inspect)(error)} `);
        }
        return node;
    }
    async getNodeAccessToken(node) {
        const tokenRequestBody = {
            username: `node-${node.id}@jfpip`,
            scope: 'internal:pipelines/buildnode/project/'.concat('node.projectId', ':r,w,d'),
            refreshable: false,
            audience: this.properties.rtServiceId,
            expiresIn: oneYearInSeconds,
            grantType: 'client_credentials'
        };
        const tokenResponse = await this.accessClient.createToken({}, tokenRequestBody);
        return tokenResponse.accessToken;
    }
    async updateNodeBootStrapToken(node, token) {
        return this.nodeDao.updateByInstance(node, { systemPropertyBag: { token: token } });
    }
    async sendWWWSignal(node) {
        const signalPayload = {
            eventType: `node.created`,
            rooms: `v3/projects/${node.projectId}`,
            payload: { id: node.id }
        };
        return this.publisher.publishMessage(`www.signals`, signalPayload);
    }
}
exports.NodeService = NodeService;
//# sourceMappingURL=NodeService.js.map