"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const access_nodejs_client_1 = require("@jfrog/access-nodejs-client");
const NodeDao_1 = require("../NodeDao");
const NodeService_1 = require("../NodeService");
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
const client = new amqp_client_1.AmqpClient('', '', { amqpUrl: '' }, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publisher = new amqp_client_1.AmqpPublisher(client, '', amqp_client_1.publisherOptions, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce();
jest.spyOn(configuration_1.ConfigFactory, 'config', 'get').mockImplementation(() => {
    return { rtRouterJoinKey: 'abcd1234' };
});
jest
    .spyOn(access_nodejs_client_1.AccessClient.prototype, 'createToken')
    .mockImplementation((_context, _tokenRequest) => {
    const response = { accessToken: 'testtoken' };
    return Promise.resolve(response);
});
describe('Test NodeService #createNode', () => {
    afterEach(() => {
        publishMessageMock.mockClear();
    });
    it('should be able to create node based on node create attributes', async () => {
        const nodeCreateAttributes = {
            friendlyName: 'test',
            projectId: 1,
            nodePoolId: 1
        };
        const dao = new NodeDao_1.NodeDao();
        const expectedValue = { friendlyName: 'test', projectId: 1, nodePoolId: 1 };
        const createMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'create').mockResolvedValueOnce(expectedValue);
        const updateMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'updateByInstance').mockResolvedValueOnce(expectedValue);
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.createNode(nodeCreateAttributes);
        expect(dao.create).toHaveBeenCalled();
        expect(gotResponse).toBe(expectedValue);
        expect(publisher.publishMessage).toHaveBeenCalled();
        createMock.mockClear();
        updateMock.mockClear();
    });
});
describe('Test NodeService #getRunningNodes', () => {
    it('should return nodes with active status', async () => {
        const dao = new NodeDao_1.NodeDao();
        const expectedResponse = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success }
        ];
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockResolvedValueOnce(expectedResponse);
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.getRunningNodes(1);
        expect(dao.findAll).toHaveBeenCalled();
        expect(gotResponse).toBe(expectedResponse);
        findAllMock.mockClear();
    });
    it('should return null if no active nodes found', async () => {
        const dao = new NodeDao_1.NodeDao();
        const findAllMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'findAll').mockImplementationOnce(() => Promise.resolve([]));
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = await service.getRunningNodes(1);
        expect(dao.findAll).toHaveBeenCalled();
        expect(gotResponse).toStrictEqual([]);
        findAllMock.mockClear();
    });
});
describe('Test NodeService #filterActiveNodes', () => {
    it('should return only nodes status code is not NodeStatus.cached', () => {
        const expectedResponse = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
            { friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success }
        ];
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = service.filterActiveNodes(expectedResponse);
        expect(gotResponse.length).toBe(2);
    });
});
describe('Test NodeService #filterIdleNodes', () => {
    it('should return only nodes whose nodepool idle interval has surpassed', () => {
        const expectedResponse = [
            {
                friendlyName: 'test',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date()
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.waiting,
                statusLastUpdatedAt: new Date()
            },
            {
                friendlyName: 'test2',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date()
            }
        ];
        const service = new NodeService_1.NodeService(publisher);
        const pool = { id: 1, nodeIdleIntervalInMins: 15 };
        const gotResponse = service.filterIdleNodes(pool, expectedResponse);
        expect(gotResponse.length).toBe(0);
    });
});
describe('Test NodeService #filterCachedNodes', () => {
    it('should return only nodes status code is  NodeStatus.cached', () => {
        const expectedResponse = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
            { friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success }
        ];
        const service = new NodeService_1.NodeService(publisher);
        const gotResponse = service.filterCachedNodes(expectedResponse);
        expect(gotResponse.length).toBe(1);
    });
});
describe('Test NodeService #applyNodeToStoppedState', () => {
    it('should change the node status to stopped', async () => {
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped };
        const dao = new NodeDao_1.NodeDao();
        const updateByInstanceMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'updateByInstance').mockResolvedValueOnce(node);
        const service = new NodeService_1.NodeService(publisher);
        const expectedResponse = await service.applyNodeToStoppedState(node);
        expect(dao.updateByInstance).toHaveBeenCalled();
        expect(expectedResponse.statusCode).toBe(NodeService_1.NodeStatus.stopped);
        updateByInstanceMock.mockClear();
    });
});
describe('Test NodeService #applyNodeToCachedState', () => {
    it('should change node status to cached', async () => {
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached };
        const dao = new NodeDao_1.NodeDao();
        const updateByInstanceMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'updateByInstance').mockResolvedValueOnce(node);
        const service = new NodeService_1.NodeService(publisher);
        const expectedResponse = await service.applyNodeToCachedState(node);
        expect(dao.updateByInstance).toHaveBeenCalled();
        expect(expectedResponse.statusCode).toBe(NodeService_1.NodeStatus.cached);
        updateByInstanceMock.mockClear();
    });
});
describe('Test NodeService #applyNodeWaitingState', () => {
    it('should change node status to cached to waiting', async () => {
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const dao = new NodeDao_1.NodeDao();
        const updateByInstanceMock = jest.spyOn(NodeDao_1.NodeDao.prototype, 'updateByInstance').mockResolvedValueOnce(node);
        const service = new NodeService_1.NodeService(publisher);
        const expectedResponse = await service.applyNodeWaitingState(node);
        expect(dao.updateByInstance).toHaveBeenCalled();
        expect(expectedResponse.statusCode).toBe(NodeService_1.NodeStatus.waiting);
        updateByInstanceMock.mockClear();
    });
});
//# sourceMappingURL=NodeService.test.js.map